<?php

namespace Drupal\fileslog\Logger;

use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LogMessageParserInterface;
use Drupal\Core\Logger\RfcLoggerTrait;
use Psr\Log\LoggerInterface;

/**
 * Redirects logging messages to fileslog.
 */
class FilesLog implements LoggerInterface {
  use RfcLoggerTrait;

  /**
   * The directory to put logs in.
   *
   * @var string
   */
  public const DIRECTORY = 'private://logs';

  /**
   * The message's placeholders parser.
   *
   * @var \Drupal\Core\Logger\LogMessageParserInterface
   */
  protected LogMessageParserInterface $parser;

  /**
   * Constructs a fileslog object.
   *
   * @param \Drupal\Core\Logger\LogMessageParserInterface $parser
   *   The parser to use when extracting message variables.
   */
  public function __construct(LogMessageParserInterface $parser) {
    $this->parser = $parser;
  }

  /**
   * {@inheritdoc}
   */
  public function log($level, $message, array $context = []) {
    global $base_url;

    // Populate the message placeholders and then replace them in the message.
    $message_placeholders = $this->parser->parseMessagePlaceholders($message, $context);
    $message = empty($message_placeholders) ? $message : strtr($message, $message_placeholders);

    $channel = $context['channel'];
    $entry = json_encode([
      'base_url' => $base_url,
      'timestamp' => $context['timestamp'],
      'type' => $channel,
      'ip' => $context['ip'],
      'request_uri' => $context['request_uri'],
      'referer' => $context['referer'],
      'severity' => $level,
      'uid' => $context['uid'],
      'link' => strip_tags($context['link']),
      'message' => strip_tags($message),
    ], JSON_PRETTY_PRINT);

    $this->filesLogWrapper($level, $channel, $entry);
  }

  /**
   * A fileslog wrapper to make fileslog functionality testable.
   *
   * @param int $level
   *   The log's severity.
   * @param string $channel
   *   The log's channel.
   * @param string $entry
   *   The entry to store in the log..
   */
  protected function filesLogWrapper(int $level, string $channel, string $entry) {
    /**
     * We have to add it this way to prevent circular reference errors.
     *
     * @var \Drupal\Core\File\FileSystemInterface $filesystem
     */
    $filesystem = \Drupal::service('file_system');

    // @todo Implement channel somehow so we can filter on it easily.
    $directory = static::DIRECTORY;
    $filesystem->prepareDirectory($directory, FileSystemInterface::CREATE_DIRECTORY);
    $filesystem->prepareDirectory($directory, FileSystemInterface::MODIFY_PERMISSIONS);
    $filesystem->saveData($entry, $directory . '/' . microtime(TRUE) . '-' . $level . '.json');
  }

}
